<?php

/**
 * @file
 * Contains Drupal\config_share\ConfigShareConfigInstaller.
 */

namespace Drupal\config_share;

use Drupal\Core\Config\ConfigInstaller;
use Drupal\Core\Config\ExtensionInstallStorage;
use Drupal\Core\Config\StorageInterface;

class ConfigShareConfigInstaller extends ConfigInstaller {

  /**
   * Extension sub-directory containing shared configuration for installation.
   */
  const CONFIG_SHARED_DIRECTORY = 'config/shared';

  /**
   * Overrides \Drupal\Core\Config\ConfigInstaller::getConfigToCreate().
   *
   * When extensions are installed, analyze their configuration
   * and create any shared configuration entity dependencies.
   *
   * @param StorageInterface $storage
   *   The configuration storage to read configuration from.
   * @param string $collection
   *  The configuration collection to use.
   * @param string $prefix
   *   (optional) Limit to configuration starting with the provided string.
   *
   * @return array
   *   An array of configuration data read from the source storage keyed by the
   *   configuration object name.
   */
  protected function getConfigToCreate(StorageInterface $storage, $collection, $prefix = '', StorageInterface $profile_storage = NULL) {
    // Determine if we have configuration to create.
    $config_to_create = parent::getConfigToCreate($storage, $collection, $prefix, $profile_storage);

    if (!empty($config_to_create)) {
      $existing_config = $this->getActiveStorages()->listAll();

      // Search the install profile's shared configuration too.
      $shared_storage = new ExtensionInstallStorage($this->getActiveStorages($collection), self::CONFIG_SHARED_DIRECTORY, $collection, TRUE);

      // Find all shared configuration for enabled modules.
      $shared_config = $shared_storage->listAll();
      $shared_config = array_diff($shared_config, $existing_config);
      $shared_config_data = $shared_storage->readMultiple($shared_config);

      // Determine shared configuration to create.
      $shared_config_to_create = array();

      // Find dependencies of configuration items to be created.
      foreach ($config_to_create as $config_entity) {
        $this->mergeSharedDependencies($config_entity, $shared_config_to_create, $shared_config_data);
      }

      $config_to_create = array_merge($shared_config_to_create, $config_to_create);
    }

    return $config_to_create;
  }

  /**
   * Merges in shared configuration to be created based on dependency
   * information.
   *
   * @param array $config_entity
   *   A configuration entity's data.
   * @param array &$shared_config_to_create
   *   An array of shared configuration to be created, keyed by the
   *   configuration object name; passed by reference.
   * @param array $shared_config_data
   *   An array of available shared configuration data read from the source
   *   storage keyed by the configuration object name.
   */
  protected function mergeSharedDependencies($config_entity, &$shared_config_to_create, $shared_config_data) {
    if (isset($config_entity['dependencies']['config'])) {
      $shared_config_to_add = array_intersect(array_keys($shared_config_data), $config_entity['dependencies']['config']);
      foreach ($shared_config_to_add as $name) {
        $shared_config_to_create[$name] = $shared_config_data[$name];
        // Shared configuration may itself depend on other shared configuration.
        $this->mergeSharedDependencies($shared_config_data[$name], $shared_config_to_create, $shared_config_data);
      }
    }
  }
}
